@@ -5,12 +5,13 @@
 
 
 from pyts2.time import TSInstant, TimeFilter
-from pyts2.timestream import TimeStream, TimestreamFile
+from pyts2.timestream import TimeStream, TimestreamFile, InMemoryTimeStream
 from pyts2._version import get_versions
 __version__ = get_versions()['version']
 del get_versions
 
 __all__ = [
+    'InMemoryTimeStream',
     'TimestreamFile',
     'TSInstant',
     'TimeStream',
