@@ -22,6 +22,7 @@ from pyts2.removalist import Removalist
 from os.path import dirname, basename, splitext, getsize, realpath
 from sys import stdout, stderr, stdin, exit  # noqa
 from csv import DictWriter
+from functools import partial
 import datetime
 import argparse
 import multiprocessing as mp
@@ -31,6 +32,7 @@ from shlex import quote
 import os
 import shutil
 import sys
+import json
 
 
 def getncpu():
@@ -50,6 +52,9 @@ def valid_time(s):
     except ValueError:
         raise argparse.ArgumentTypeError(f"Not a valid time in H-M-S form: '{s}'.")
 
+def filter_mean_luminance(img, luminance=20):
+    return img.report["ImageMean_L"] >= luminance
+
 
 @click.group()
 def tstk_main():
@@ -97,6 +102,7 @@ def bundle(force, informat, bundle, input, output):
               help="Telegraf reporting metric name")
 @click.argument("input")
 def audit(input, output, telegraf_host, telegraf_port, telegraf_metric, ncpus=1, informat=None):
+    from pyts2.pipeline.telegraf import TelegrafRecordStep
     if output is None and telegraf_host is None:
         print("ERROR: must give one of --output or --telegraf-host")
         sys.exit(1)
@@ -122,6 +128,10 @@ def audit(input, output, telegraf_host, telegraf_port, telegraf_metric, ncpus=1,
             if output is not None:
                 if pipe.n % 1000 == 0:
                     pipe.report.save(output)
+    except Exception as exc:
+        print(f"ERROR: {exc.__class__.__name__} {str(exc)}")
+        if stderr.isatty():
+            traceback.print_exc(file=stderr)
     finally:
         if output is not None:
             pipe.report.save(output)
@@ -164,6 +174,10 @@ def downsize(input, output, ncpus, informat, outformat, size, bundle, mode, flat
     outts = TimeStream(output, format=outformat, bundle_level=bundle, add_subsecond_field=True, flat_output=flat)
     try:
         pipe.process_to(ints, outts, ncpus=ncpus)
+    except Exception as exc:
+        print(f"ERROR: {exc.__class__.__name__} {str(exc)}")
+        if stderr.isatty():
+            traceback.print_exc(file=stderr)
     finally:
         click.echo(f"{mode} {input}:{informat} to {output}:{outformat}, found {pipe.n} files")
 
@@ -189,40 +203,84 @@ def downsize(input, output, ncpus, informat, outformat, size, bundle, mode, flat
               help="Level at which to bundle downsized images.")
 @click.option("--audit-output", "-a", type=Path(writable=True), default=None,
               help="Audit log output TSV. If given, input images will be audited, with the log saved here.")
-def ingest(input, informat, output, bundle, ncpus, downsized_output, downsized_size, downsized_bundle, audit_output):
+@click.option("--centrecropped-output", "--co", default=None,
+              help="Output a centrecropped copy of the images here")
+@click.option("--centrecropped-size", "--cs", default='720x',
+              help="Downsized output size. Use ROWSxCOLS. One of ROWS or COLS can be omitted to keep aspect ratio.")
+@click.option("--centrecropped-bundle", "--cb", type=Choice(TimeStream.bundle_levels), default="none",
+              help="Level at which to bundle centrecropped images.")
+@click.option("--min-mean-luminance", "--ml", type=float, default=None,
+              help="Don't keep originals with mean luminance < X. (black = 0 <= L <= 100 = white).")
+@click.option("--truncate-time", type=int, default=None, metavar="MINUTES",
+              help="Truncate time to N-mintue intervals.")
+@click.option("--NUKE", is_flag=True, default=False,
+              help="DELETE file UNSAFELY as it finishes processsing")
+def ingest(input, informat, output, bundle, ncpus,
+           downsized_output, downsized_size, downsized_bundle, audit_output,
+           centrecropped_output, centrecropped_size, centrecropped_bundle,
+           truncate_time, nuke, min_mean_luminance):
     ints = TimeStream(input, format=informat)
     outts = TimeStream(output, bundle_level=bundle)
 
-    steps = [WriteFileStep(outts)]
+    pipe = TSPipeline()
 
-    # if downsized_output is not None or audit_output is not None:
-    #    steps.append(DecodeImageFileStep())
+    if truncate_time is not None:
+        pipe.add_step(TruncateTimeStep(truncate_time))
+
+    if downsized_output is not None or audit_output is not None or centrecropped_output is not None:
+        pipe.add_step(DecodeImageFileStep())
 
     if audit_output is not None:
         audit_pipe = TSPipeline(
             FileStatsStep(),
             CalculateEVStep(),
-            DecodeImageFileStep(),
             ImageMeanColourStep(),
             ScanQRCodesStep(),
         )
-        steps.append(audit_pipe)
+        pipe.add_step(audit_pipe)
 
     if downsized_output is not None:
         downsized_ts = TimeStream(downsized_output, bundle_level=downsized_bundle, add_subsecond_field=True)
         downsize_pipeline = TSPipeline(
-            DecodeImageFileStep(),
             ResizeImageStep(geom=downsized_size),
             EncodeImageFileStep(format="jpg"),
             WriteFileStep(downsized_ts),
+            ClearFileObjectStep(),
         )
-        steps.append(downsize_pipeline)
+        pipe.add_step(TeeStep(downsize_pipeline))
 
-    pipe = TSPipeline(*steps)
+    if centrecropped_output is not None:
+        centrecropped_ts = TimeStream(centrecropped_output, bundle_level=centrecropped_bundle, add_subsecond_field=True)
+        centrecrop_pipeline = TSPipeline(
+            CropCentreStep(geom=centrecropped_size),
+            EncodeImageFileStep(format="jpg"),
+            WriteFileStep(centrecropped_ts),
+            ClearFileObjectStep(),
+        )
+        pipe.add_step(TeeStep(centrecrop_pipeline))
 
+    if min_mean_luminance is None:
+        # set value to -inf so that all images pass
+        min_mean_luminance = float("-inf")
+    filter_on_luminance = partial(filter_mean_luminance, luminance=min_mean_luminance)
+    pipe.add_step(ConditionalStep(filter_on_luminance, WriteFileStep(outts)))
+
+    if nuke:
+        pipe.add_step(UnsafeNuker())
+
+    pipe.add_step(ClearFileObjectStep())
+
+    print("Running the following pipeline", file=stderr)
+    print(repr(pipe), file=stderr)
+    print("="*80, file=stderr)
+    print("", file=stderr)
     try:
         for image in pipe.process(ints, ncpus=ncpus):
             pass
+    except Exception as exc:
+        print(f"ERROR: {exc.__class__.__name__} {str(exc)}")
+        if stderr.isatty():
+            traceback.print_exc(file=stderr)
     finally:
         pipe.finish()
         if audit_output is not None:
@@ -231,6 +289,7 @@ def ingest(input, informat, output, bundle, ncpus, downsized_output, downsized_s
         click.echo(f"Ingested {input}{ifmt} to {output}, found {pipe.n} files")
         sys.exit(pipe.retcode)
 
+
 @tstk_main.command()
 @click.option("--input", "-i", default=stdin, type=click.File("r"),
               help="file of file names to input (default stdin).")
@@ -242,54 +301,113 @@ def ingest(input, informat, output, bundle, ncpus, downsized_output, downsized_s
               help="Archival bundled TimeStream")
 @click.option("--bundle", "-b", type=Choice(TimeStream.bundle_levels), default="none",
               help="Level at which to bundle files.")
-@click.option("--downsized-output", "-s", default=None,
+@click.option("--recoded-output", "--ro", type=str, default=None,
+              help="Output timestream for recoded import images")
+@click.option("--recoded-format", "--rf", type=str, default=None,
+              help="File format of  images")
+@click.option("--recoded-bundle", "--rb", type=Choice(TimeStream.bundle_levels), default="none",
+              help="Level at which to bundle recoded images")
+@click.option("--downsized-output", "--do", default=None,
               help="Output a downsized copy of the images here")
-@click.option("--downsized-size", "-S", default='720x',
+@click.option("--downsized-size", "--ds", default='720x',
               help="Downsized output size. Use ROWSxCOLS. One of ROWS or COLS can be omitted to keep aspect ratio.")
-@click.option("--downsized-bundle", "-B", type=Choice(TimeStream.bundle_levels), default="root",
+@click.option("--downsized-bundle", "--db", type=Choice(TimeStream.bundle_levels), default="none",
               help="Level at which to bundle downsized images.")
+@click.option("--centrecropped-output", "--co", default=None,
+              help="Output a centrecropped copy of the images here")
+@click.option("--centrecropped-size", "--cs", default='720x',
+              help="Downsized output size. Use ROWSxCOLS. One of ROWS or COLS can be omitted to keep aspect ratio.")
+@click.option("--centrecropped-bundle", "--cb", type=Choice(TimeStream.bundle_levels), default="none",
+              help="Level at which to bundle centrecropped images.")
+@click.option("--min-mean-luminance", "--ml", type=float, default=None,
+              help="Don't keep originals with mean luminance < X. (black = 0 <= L <= 100 = white).")
 @click.option("--telegraf-host", default="localhost",
               help="Telegraf reporting host")
 @click.option("--telegraf-port", default=8092,
               help="Telegraf reporting port")
 @click.option("--telegraf-metric", default='tstk_live_ingest',
               help="Telegraf reporting metric name")
+@click.option("--telegraf-additional-tags", default=None, type=str,
+              help="Json-coded addition tags that are passed as metric tags.")
 @click.option("--NUKE", is_flag=True, default=False,
               help="DELETE file UNSAFELY as it finishes processsing")
-def liveingest(input, informat, output, bundle, downsized_output, downsized_size, downsized_bundle, telegraf_host, telegraf_port, telegraf_metric, inotify_watch, nuke):
+@click.option("--truncate-time", type=int, default=None, metavar="MINUTES",
+              help="Truncate time to N-mintue intervals.")
+def liveingest(input, informat, output, bundle, inotify_watch, nuke, min_mean_luminance, truncate_time,
+               downsized_output, downsized_size, downsized_bundle,
+               recoded_output, recoded_format, recoded_bundle,
+               centrecropped_output, centrecropped_size, centrecropped_bundle,
+               telegraf_host, telegraf_port, telegraf_metric, telegraf_additional_tags,
+               ):
+    from pyts2.pipeline.telegraf import TelegrafRecordStep
+    ifmt = f"{informat}s" if informat is not None else "images"
+    click.echo(f"Begin live ingest of {ifmt} to {output}...")
+
+
+    if telegraf_additional_tags is not None:
+        telegraf_additional_tags = json.loads(telegraf_additional_tags)
+    else:
+        telegraf_additional_tags = {}
 
     ints = TimeStream(format=informat)
     outts = TimeStream(output, bundle_level=bundle)
 
     pipe = TSPipeline()
-    pipe.add_step(WriteFileStep(outts))
 
-    audit_pipe = TSPipeline(
-        FileStatsStep(),
-        DecodeImageFileStep(),
-        ImageMeanColourStep(),
-        ScanQRCodesStep(),
-        TelegrafRecordStep(
-            metric_name=telegraf_metric,
-            telegraf_host=telegraf_host,
-            telegraf_port=telegraf_port,
-        ),
-    )
-    pipe.add_step(audit_pipe)
+    pipe.add_step(FileStatsStep())
+    pipe.add_step(DecodeImageFileStep())
+    pipe.add_step(ImageMeanColourStep())
+    pipe.add_step(CalculateEVStep())
+    pipe.add_step(ScanQRCodesStep())
+
+    pipe.add_step(TelegrafRecordStep(
+        metric_name=telegraf_metric,
+        telegraf_host=telegraf_host,
+        telegraf_port=telegraf_port,
+        tags=telegraf_additional_tags,
+    ))
 
     if downsized_output is not None:
         downsized_ts = TimeStream(downsized_output, bundle_level=downsized_bundle, add_subsecond_field=True)
         downsize_pipeline = TSPipeline(
-            DecodeImageFileStep(),
             ResizeImageStep(geom=downsized_size),
             EncodeImageFileStep(format="jpg"),
             WriteFileStep(downsized_ts),
+            ClearFileObjectStep(),
         )
         pipe.add_step(TeeStep(downsize_pipeline))
 
+    if centrecropped_output is not None:
+        centrecropped_ts = TimeStream(centrecropped_output, bundle_level=centrecropped_bundle, add_subsecond_field=True)
+        centrecrop_pipeline = TSPipeline(
+            DecodeImageFileStep(),
+            CropCentreStep(geom=centrecropped_size),
+            EncodeImageFileStep(format="jpg"),
+            WriteFileStep(centrecropped_ts),
+            ClearFileObjectStep(),
+        )
+        pipe.add_step(TeeStep(centrecrop_pipeline))
+
+    if min_mean_luminance is None:
+        # set value to -inf so that all images pass
+        min_mean_luminance = float("-inf")
+    filter_on_luminance = partial(filter_mean_luminance, luminance=min_mean_luminance)
+    pipe.add_step(ConditionalStep(filter_on_luminance, WriteFileStep(outts)))
+
+    if recoded_output is not None:
+        recoded_ts = TimeStream(recoded_output, bundle_level=recoded_bundle, add_subsecond_field=True)
+        recode_pipeline = TSPipeline(
+            EncodeImageFileStep(format=recoded_format),
+            WriteFileStep(recoded_ts),
+            ClearFileObjectStep(),
+        )
+        pipe.add_step(TeeStep(recode_pipeline))
+
     if nuke:
         pipe.add_step(UnsafeNuker())
 
+    pipe.add_step(ClearFileObjectStep())
+
     try:
         if inotify_watch is not None:
             instream = ints.from_inotify(inotify_watch)
@@ -297,12 +415,15 @@ def liveingest(input, informat, output, bundle, downsized_output, downsized_size
             instream = ints.from_fofn(input)
         for image in instream:
             image = pipe.process_file(image)
-            click.echo(f"{image.instant} Done")
+            click.echo(f"{image.filename} Done")
             pipe.n += 1
+    except Exception as exc:
+        print(f"ERROR: {exc.__class__.__name__} {str(exc)}")
+        if stderr.isatty():
+            traceback.print_exc(file=stderr)
     finally:
         pipe.finish()
-        ifmt = f":{informat}" if informat is not None else ""
-        click.echo(f"Ingested {input.name}{ifmt} to {output}, found {pipe.n} files")
+        click.echo(f"Ingested {ifmt} to {output}, found {pipe.n} files")
 
 
 @tstk_main.command()
@@ -389,8 +510,8 @@ def verify(ephemerals, resource, informat, force_delete, rm_script, move_dest, p
               type=Choice(["colsright", "colsleft", "rowsdown", "rowsup"]),
               help="Order in which images are taken (cols or rows, left orright)")
 # time
-@click.option("--truncate-time", type=str, default=None, metavar="TIME",
-              help="Truncate time to TIME")
+@click.option("--truncate-time", type=int, default=None, metavar="MINUTES",
+              help="Truncate time to N-mintue intervals.")
 # audit
 @click.option("--audit-output", "-a", type=Path(writable=True), default=None,
               help="Audit log output TSV. If given, input images will be audited, with the log saved here.")
@@ -479,6 +600,10 @@ def gvmosaic(input, informat, dims, order, audit_output, composite_bundling,
     try:
         for image in pipe.process(ints):
             pass
+    except Exception as exc:
+        print(f"ERROR: {exc.__class__.__name__} {str(exc)}")
+        if stderr.isatty():
+            traceback.print_exc(file=stderr)
     finally:
         pipe.finish()
         if audit_output is not None:
@@ -588,6 +713,8 @@ def findpairs(input, rm_script, move_dest, force_delete):
 @tstk_main.command()
 @click.option("--output", "-o", type=Path(writable=True),
               help="Output video file name")
+@click.option("--segmented-by", default=None, type=Choice(("year", "month", "day")),
+              help="Produce one video per time segment (year/month/day). --output is then a prefix.")
 @click.option("--ffmpeg-path", "-b", default="ffmpeg",
               help="ffmpeg command/path")
 @click.option("--ffmpeg-args", default=None,
@@ -602,23 +729,33 @@ def findpairs(input, rm_script, move_dest, force_delete):
 @click.option("--informat", "-F", default=None,
               help="Input image format (use extension as lower case for raw formats)")
 @click.argument("input")
-def video(input, output, ffmpeg_path, ffmpeg_args, framerate, scaling, ncpus, informat):
+def video(input, output, ffmpeg_path, ffmpeg_args, framerate, scaling, ncpus, informat, segmented_by):
     """Creates a standard timelapse video from timestream"""
+    from pyts2.pipeline.video import VideoEncoder, ImageWatermarker, SegementedVideoEncoder
 
-    from pyts2.pipeline.video import VideoEncoder, ImageWatermarker
-
-    # ffmpeg_command = ffmpeg_path+' '+ffmpeg_arg
     ints = TimeStream(input, format=informat)
     pipe = TSPipeline(
         DecodeImageFileStep(),
         ImageWatermarker(),
         EncodeImageFileStep(format="jpg"),
-        VideoEncoder(output, ffmpeg_args=ffmpeg_args, ffmpeg_path=ffmpeg_path,
-            rate=framerate, threads=ncpus, scaling=scaling),
     )
+    if segmented_by is None:
+        pipe.add_step(
+            VideoEncoder(output, ffmpeg_args=ffmpeg_args, ffmpeg_path=ffmpeg_path,
+                rate=framerate, threads=ncpus, scaling=scaling),
+        )
+    else:
+        pipe.add_step(
+            SegementedVideoEncoder(output, segmented_by=segmented_by, ffmpeg_args=ffmpeg_args, ffmpeg_path=ffmpeg_path,
+                rate=framerate, threads=ncpus, scaling=scaling),
+        )
     try:
         for image in pipe.process(ints, ncpus=1):
             pass
+    except Exception as exc:
+        print(f"ERROR: {exc.__class__.__name__} {str(exc)}")
+        if stderr.isatty():
+            traceback.print_exc(file=stderr)
     finally:
         pipe.finish()
         sys.exit(pipe.retcode)
