@@ -4,19 +4,23 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-
-from tqdm import tqdm
-import msgpack
-import time
-import copy
-from collections import defaultdict
-import csv
-from concurrent.futures import ProcessPoolExecutor
-from os import path as op
-import re
-from sys import stderr
-import traceback
 import gc
+import traceback
+from sys import stderr
+import re
+from os import path as op
+from concurrent.futures import ProcessPoolExecutor
+import csv
+from collections import defaultdict
+import copy
+import time
+import msgpack
+
+try:
+    from tqdm import tqdm
+    _HAVE_TQDM = True
+except ImportError:
+    _HAVE_TQDM = False
 
 csv.register_dialect('tsv',
                      delimiter='\t',
@@ -49,13 +53,19 @@ class TSPipeline(object):
     Core TimeStream pipeline runner.
     """
 
-    def __init__(self, *args, reporter=None, record_step_times=False):
+    def __init__(self, *args, reporter=None, record_step_times=False, step_id=None):
         r"""
         :param \*args: Pipeline steps to execute
         :type \*args: :class:`.PipelineStep`
         :param reporter: Object to hold reports
         :type reporter: :class:`.ResultRecorder`
+        :param record_step_times: whether to record the time take for each step in the file report
+        :type record_step_times: bool
+        :param step_id: (options) when record_step_times is True, what to record the step name as
+        :type step_id: str
         """
+        if step_id:
+            self.step_id = step_id
         self.record_step_times = record_step_times
         self.retcode = 0
         self.n = 0
@@ -107,6 +117,12 @@ class TSPipeline(object):
                     raise
             if self.record_step_times:
                 fname = f'StepTime_{step.__class__.__name__}'
+                if hasattr(step, "step_id"):
+                    fname += f".{step.step_id}"
+                elif hasattr(self, "step_id"):
+                    fname += f".{self.step_id}"
+
+                # aggregate the step times that already exist as totals
                 if fname in file.report:
                     file.report[fname] += time.time() - t
                 else:
@@ -125,17 +141,32 @@ class TSPipeline(object):
         :param progress: Unimplemented
         :type progress: bool
         """
+
         try:
             if ncpus > 1:
                 with ProcessPoolExecutor(max_workers=ncpus) as executor:
-                    for file in tqdm(executor.map(self.process_file, input_stream), unit=" files"):
-                        if file is None:
-                            continue
-                        self.report.record(file.instant, **file.report)
-                        self.n += 1
-                        yield file
+                    try:
+                        executor_iter = executor.map(self.process_file, input_stream)
+                        if _HAVE_TQDM:
+                            executor_iter = tqdm(executor.map(self.process_file, input_stream), unit=" files")
+                        for file in executor_iter:
+                            if file is None:
+                                continue
+                            self.report.record(file.instant, **file.report)
+                            self.n += 1
+                            yield file
+                    except ImportError:
+                        for file in executor.map(self.process_file, input_stream):
+                            if file is None:
+                                continue
+                            self.report.record(file.instant, **file.report)
+                            self.n += 1
+                            yield file
             else:
-                for file in input_stream:
+                iterable_stream = input_stream
+                if _HAVE_TQDM:
+                    iterable_stream = tqdm(input_stream)
+                for file in iterable_stream:
                     file = self.process_file(file)
                     if file is None:
                         continue
@@ -305,15 +336,18 @@ class ResultRecorderStep(PipelineStep):
 class TeeStep(PipelineStep):
     """Execute another step or pipeline with no side effects on each ``file``"""
 
-    def __init__(self, other_pipeline):
+    def __init__(self, other_pipeline, update_report=False):
         """
         :param other_pipeline: Pipeline to process this file, branching off from this point
         :type other_pipeline: :class:`pyts2.timestream.TimestreamFile`
         """
         self.pipe = other_pipeline
+        self.update_report = update_report
 
     def process_file(self, file):
-        self.pipe.process_file(copy.deepcopy(file))
+        file2 = self.pipe.process_file(copy.deepcopy(file))
+        if self.update_report:
+            file.report.update(file2.report)
         return file
 
     def __repr__(self):
