@@ -5,15 +5,12 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import numpy as np
-import skimage as ski
 
 from .base import *
 from .imageio import *
 from pyts2.utils import *
 
-import re
 import shlex
-from sys import stderr, stdout, stdin
 
 
 class GigavisionMosaicStep(PipelineStep):
@@ -86,12 +83,12 @@ class GigavisionMosaicStep(PipelineStep):
             composite_img = self.write_current()
             self.current_datetime = file.instant.datetime
             self.current_pixels = np.zeros(
-                (self.superdim[0]*self.subimgres[0],
-                 self.superdim[1]*self.subimgres[1], 3),
+                (self.superdim[0] * self.subimgres[0],
+                 self.superdim[1] * self.subimgres[1], 3),
                 dtype=np.float32
             )
 
-        row, col = index2rowcol(int(file.instant.index)-1, self.superdim[0], self.superdim[1], self.order)
+        row, col = index2rowcol(int(file.instant.index) - 1, self.superdim[0], self.superdim[1], self.order)
         top = row * self.subimgres[0]
         bottom = top + self.subimgres[0]
         left = col * self.subimgres[1]
@@ -101,8 +98,8 @@ class GigavisionMosaicStep(PipelineStep):
         if self.centrecrop is not None:
             h, w, _ = pixels.shape
             s = self.centrecrop * 0.5  # half scale factor
-            t, b = int(h*s), int(h*(1-s))
-            l, r = int(w*s), int(w*(1-s))
+            t, b = int(h * s), int(h * (1 - s))
+            l, r = int(w * s), int(w * (1 - s))
             pixels = pixels[t:b, l:r, :]
         from skimage.transform import resize
         smallpx = resize(pixels, self.subimgres, anti_aliasing=True, mode="constant", order=3)
