@@ -8,13 +8,13 @@
 from .base import PipelineStep, FatalPipelineError
 from .imageio import DecodeImageFileStep
 
+from os.path import dirname
+
 import subprocess
-from sys import stdin, stdout, stderr
-from os.path import dirname, basename
 
 import numpy as np
-import imageio
-from PIL import Image, ImageFont, ImageDraw
+from sys import stderr
+from PIL import ImageFont, ImageDraw
 
 
 def get_default_ffmpeg_cmd(rate=10, threads=1, scaling=None):
@@ -28,6 +28,7 @@ def get_default_ffmpeg_cmd(rate=10, threads=1, scaling=None):
 def get_instant(file):
     return str(file.instant)
 
+
 class ImageWatermarker(PipelineStep):
 
     def __init__(self, font=None, font_size=16, textcallback=get_instant):
@@ -38,7 +39,7 @@ class ImageWatermarker(PipelineStep):
             self.font = ImageFont.truetype(font, font_size)
         except Exception as exc:     # load default font in case of any exception
             print("Error: Couldn't locate font", font, ", using default font.\n",
-                    exc.__class__.__name__, str(exc), file=stderr)
+                  exc.__class__.__name__, str(exc), file=stderr)
             self.font = ImageFont.load_default()
 
     def process_file(self, file):
@@ -52,36 +53,35 @@ class ImageWatermarker(PipelineStep):
         left, top = (5, image.size[1] - text_height)
         text_width = self.font.getsize(text)[0]
         locus = np.asarray(image.crop((left, top, left + text_width, top + text_height)))
-        meancol = tuple(list(locus.mean(axis=(0,1)).astype(int)))
+        meancol = tuple(list(locus.mean(axis=(0, 1)).astype(int)))
         opposite = (int(locus.mean()) + 96)
         if opposite > 255:
             opposite = (int(locus.mean()) - 96)
         oppositegrey = (opposite, ) * 3
         draw = ImageDraw.Draw(image)
-        draw.rectangle((left-3, top-3, left + text_width + 3, top + text_height + 3),
+        draw.rectangle((left - 3, top - 3, left + text_width + 3, top + text_height + 3),
                        fill=meancol)
         draw.text((left, top), text, fill=oppositegrey, font=self.font)
         file.pixels = np.asarray(image)
         return file
 
 
-
 class VideoEncoder(PipelineStep):
     def __init__(self, outfile, ffmpeg_args=None, ffmpeg_path="ffmpeg", rate=10, threads=1, scaling=None):
         if ffmpeg_args is None:
             ffmpeg_args = get_default_ffmpeg_cmd(rate=rate, threads=threads, scaling=scaling)
         ffmpeg_base_command = ffmpeg_path + ' ' + ffmpeg_args + " " + outfile
         self.ffmpeg = subprocess.Popen(ffmpeg_base_command, shell=True,
-                stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-                stderr=subprocess.STDOUT)
+                                       stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                                       stderr=subprocess.STDOUT)
 
     def process_file(self, file):
         if not hasattr(file, "pixels"):
             file = DecodeImageFileStep().process_file(file)
         try:
             self.ffmpeg.stdin.write(file.content)
-        except BrokenPipeError as exc:
-            log=self.ffmpeg.stdout.read()
+        except BrokenPipeError:
+            log = self.ffmpeg.stdout.read()
             raise FatalPipelineError(f"FFMPEG failed, log follows\n{log}")
         return file
 
@@ -125,12 +125,12 @@ class SegementedVideoEncoder(PipelineStep):
                     self.ffmpeg.wait()
                 ffmpeg_cmd = f"{self.ffmpeg_base_command} {self.outprefix}_{thisseg}.mp4"
                 self.ffmpeg = subprocess.Popen(ffmpeg_cmd, shell=True,
-                        stdin=subprocess.PIPE, stdout=subprocess.PIPE,
-                        stderr=subprocess.STDOUT)
+                                               stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                                               stderr=subprocess.STDOUT)
             self.ffmpeg.stdin.write(file.content)
         except Exception as exc:
             print(str(exc))
-            log=self.ffmpeg.stdout.read()
+            log = self.ffmpeg.stdout.read()
             raise FatalPipelineError(f"FFMPEG failed, log follows\n{log}")
         return file
 
